package com.yaoyicloud.render.platform.update;

import java.io.IOException;
import java.io.InputStream;
import java.util.Map;
import java.util.Objects;

import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.protobuf.util.JsonFormat;
import com.yaoyicloud.config.CommonDataCache;
import com.yaoyicloud.config.FilerepoProperties;
import com.yaoyicloud.factory.BasicAbstractRender;
import com.yaoyicloud.message.FxyProtos;


import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.MapUtils;

/**
 * FinancialInfo渲染器
 *
 */
@Slf4j
public final class FinancialInfoNewRender extends BasicAbstractRender {
    private final FilerepoProperties filerepoProperties;
    private final CommonDataCache commonDataCache;

    public FinancialInfoNewRender(String cwd, FilerepoProperties filerepoProperties, CommonDataCache commonDataCache) {
        super(cwd);
        this.filerepoProperties = filerepoProperties;
        this.commonDataCache = commonDataCache;
    }

    @Override
    protected String getBasicPath() throws IOException {
        return filerepoProperties.getBasePath();
    }

    @Override
    protected String getReportImagePath() {
        return filerepoProperties.getReportImagePath();
    }

    /**
     * Docx 渲染
     *
     * @param info 数据
     * @param inputStream 模板内容
     * @return 本地文件目录
     * @throws IOException
     */

    public String renderDocx(String info, InputStream inputStream, String relationId, Integer counter)
        throws IOException {
        log.info("开始渲染财务模块，relationId: {}", relationId);

        // 配置POI-TL渲染器
        ConfigureBuilder builder = Configure.builder();
        builder.useSpringEL();

        builder.bind("indicators", new LoopRowIncludeStatisticsTableRenderPolicy("values"));
        builder.bind("financialDataSeq", new LoopColumnStaticTableRenderPolicy("[", "]", false, true, 2));
        builder.bind("financialCheckDetails", new LoopRowCutAndMergeFirstColTableRenderPolicy());
        builder.bind("businessRiskChecks", this.indicatorsRenderPolicyToProtobuf());
        FxyProtos.FinancialInfo.Builder financialInfoBuilder = FxyProtos.FinancialInfo.newBuilder();
        JsonFormat.parser().merge(info, financialInfoBuilder);

        FxyProtos.FinancialInfo defaultInstance = FxyProtos.FinancialInfo.getDefaultInstance();
        FxyProtos.FinancialInfo mergedProto = defaultInstance.toBuilder()
            .mergeFrom(financialInfoBuilder.build())
            .build();

        String completeJson = JsonFormat.printer()
            .includingDefaultValueFields()
            .print(mergedProto);
        ObjectMapper objectMapper = new ObjectMapper();
        Map<String, Object> data = objectMapper.readValue(completeJson, new TypeReference<Map<String, Object>>() {});
        data.replaceAll((k, v) -> v.equals("") ? "-" : v);

        fillBasicDefaultValues(data);
        Map<String, Object> commonDataCacheData = commonDataCache.getData(relationId);
        if (MapUtils.isNotEmpty(commonDataCacheData)) {
            data.putAll(commonDataCacheData);
        }
        data.put("levelInteger", counter);
        // Map<String, Object> financialSummary = (Map<String, Object>) data.get("financialSummary");
        // data.put("fiancescore", financialSummary.get("score"));
        try {
            // 渲染文档
            String resultPath = this.renderDocx(data, inputStream, builder, relationId, "financialInfo");
            log.info("渲染财务模块成功，文件路径: {}", resultPath);
            return resultPath;
        } catch (Exception e) {
            log.error("渲染财务模块失败，relationId: {}", relationId, e);
            throw new IOException("文档渲染失败", e);
        }
    }

    /**
     * 填充默认值，确保所有必要字段都存在
     */
    private void fillBasicDefaultValues(Map<String, Object> data) {

        Map<String, Object> financialSummary = (Map<String, Object>) data.get("financialSummary");
        financialSummary.replaceAll((k, v) -> v.equals("") ? "-" : v);

        Object certReceived = data.get("certReceived");
        if (Objects.equals(certReceived, "-")) {
            data.put("certReceived", "否");
        }

        Object bankLicense = data.get("bankLicense");
        if (Objects.equals(bankLicense, "-")) {
            data.put("bankLicense", "否");
        }

        Object annualInspectionResult = data.get("annualInspectionResult");
        if (Objects.equals(annualInspectionResult, "-")) {
            data.put("annualInspectionResult", "否");
        }

    }

}
